/*
 * simple synchronization example, version 3:
 *
 * threads perform I/O with delays, introducing need to ensure one-at-a-time
 *   access to System.out.
 *
 * correct synchronization, one way.
 *
 * number of threads is taken from environment variable NUM_THREADS.
 */
package csci3366sample.synch;
import csci3366sample.ThreadUtility;

public class HelloSynch3 {

    /* main method */
    public static void main(String[] args) {

        /* get number of threads from environment variable */

        int numThreads = 0;
        try {
            numThreads = ThreadUtility.getNumThreads();
        }
        catch (ThreadUtility.Exception e) {
            System.err.println(e.getMessage());
            System.exit(1);
        }

        /* create threads */

        Thread[] threads = new Thread[numThreads];

        for (int i = 0; i < numThreads; ++i) {
            threads[i] = new Thread(new Inner(i));
        }

        /* start them up */

        System.out.println("starting threads");

        for (int i = 0; i < numThreads; ++i) {
            threads[i].start();
        }

        /* wait for them to finish */

        for (int i = 0; i < numThreads; ++i) {
            try {
                threads[i].join();
            }
            catch (InterruptedException e) {
                System.err.println("this should not happen");
            }
        }

        System.out.println("threads all done");
    }

    /* inner class containing code for each thread to execute */

    private static class Inner implements Runnable {

        private int myID;

        public Inner(int myID) {
            this.myID = myID;
        }

        public void run() {
            helloWithDelay(myID);
        }

        /* "hello with delay" method (force delay in assembling/printing line) */
        private static synchronized void helloWithDelay(int threadID) {
            System.out.print("hello from thread ");
            try {
                Thread.sleep(10);
            }
            catch (InterruptedException e) {
                System.err.println("this should not happen");
            }
            System.out.println(threadID);
        }
    }
}
